"use strict";
/*
 * Copyright (c) 2021, knokbak and contributors.
 *
 * The Birb.JS Project: https://birb.js.org
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const CacheError_1 = __importDefault(require("../errors/CacheError"));
const MemoryLeakWarning_1 = __importDefault(require("../errors/MemoryLeakWarning"));
const OptionError_1 = __importDefault(require("../errors/OptionError"));
const Pair_1 = __importDefault(require("./Pair"));
class Cache {
    constructor(options) {
        this.cache = new Map();
        this.options = {
            maxSize: null,
            maxAge: null,
            checkInterval: 60,
            removeOldest: true,
        };
        options = options ?? {};
        if ('maxSize' in options) {
            if (typeof options.maxSize !== 'number') {
                throw new OptionError_1.default('maxSize [in options at 0] must be a number');
            }
            this.options.maxSize = options.maxSize ?? null;
        }
        if ('maxAge' in options) {
            if (typeof options.maxAge !== 'number') {
                throw new OptionError_1.default('maxAge [in options at 0] must be a number');
            }
            this.options.maxAge = options.maxAge ?? null;
        }
        if ('checkInterval' in options) {
            if (typeof options.checkInterval !== 'number') {
                throw new OptionError_1.default('checkInterval [in options at 0] must be a number');
            }
            this.options.checkInterval = options.checkInterval ?? 60;
        }
        if ('removeOldest' in options) {
            if (typeof options.removeOldest !== 'boolean') {
                throw new OptionError_1.default('removeOldest [in options at 0] must be a boolean');
            }
            this.options.removeOldest = options.removeOldest ?? true;
        }
        setInterval(() => {
            for (let [key, value] of this.cache.entries()) {
                if (this.options.maxAge && Date.now() - value.timestamp > this.options.maxAge) {
                    this.remove(key);
                }
            }
        }, this.options.checkInterval * 1000);
    }
    /**
     * Get a value from the cache. If the key is not found
     * in the cache, `null` is returned.
     *
     * @param {string} key The key to get the value for.
     * @returns {any | null} The value for the key, or `null` if the key is not found.
     * @public
     */
    get(key) {
        let entry = this.cache.get(key);
        return entry?.v ?? null;
    }
    /**
     * Set a value in the cache.
     *
     * @param {string} key The key of the entry.
     * @param {any} value The value to set it to.
     * @returns {Cache} The updated cache.
     * @public
     */
    set(key, value, options) {
        let push = {
            t: Date.now(),
            v: value,
        };
        this.makeSpace(options);
        this.cache.set(key, push);
        return this;
    }
    /**
     * Check if the cache contains a key.
     *
     * @param {string} key The key of the entry.
     * @returns {boolean} `true` if the key is found, `false` otherwise.
     * @public
     */
    has(key) {
        return this.cache.has(key);
    }
    /**
     * Remove a key from the cache.
     *
     * @param {string} key The key of the entry.
     * @returns {Cache} The updated cache.
     * @public
     */
    remove(key) {
        this.cache.delete(key);
        return this;
    }
    /**
     * Get the amount of entries in the cache.
     *
     * @returns {number} The size of the cache.
     * @public
     */
    size() {
        return this.cache.size;
    }
    /**
     * Get an object of the entries in the cache. The
     * key is the key of the entry, and the value is
     * the value of the entry.
     *
     * @returns {{ [key: string]: any }} The array.
     * @public
     */
    entries() {
        let entries = {};
        for (let [key, value] of this.cache.entries()) {
            entries[key] = value.v;
        }
        return entries;
    }
    /**
     * Get an array of the entries in the cache.
     *
     * @returns {Pair<string, any>[]} The array.
     * @public
     */
    array() {
        let entries = [];
        for (let [key, value] of this.cache.entries()) {
            entries.push(new Pair_1.default(key, value.v));
        }
        return entries;
    }
    /**
     * Clear the cache and its contents.
     *
     * @returns {Cache} The updated cache.
     * @public
     */
    clear() {
        this.cache.clear();
        return this;
    }
    /**
     * Make space in the cache for a new entry. You
     * **probably shouldn't** run this method directly
     * unless you know **EXACTLY** what you're doing.
     *
     * @returns {void}
     * @public
     */
    makeSpace(options) {
        options = options ?? {};
        options.maxAge = options.maxAge || this.options.maxAge || null;
        options.removeOldest = options.removeOldest ?? this.options.removeOldest ?? true;
        options.allowOverflow = options.allowOverflow ?? false;
        if (this.options.maxSize && this.cache.size >= this.options.maxSize) {
            if (options.allowOverflow) {
                process.emitWarning(new MemoryLeakWarning_1.default('possible memory leak detected: overflow for cache has been permitted by options provided'));
                this.options.maxSize++;
            }
            else if (options.removeOldest) {
                let oldest = this.cache.entries().next().value;
                this.remove(oldest[0]);
            }
            else {
                throw new CacheError_1.default('cannot write to cache; cache is full');
            }
        }
    }
}
exports.default = Cache;
