"use strict";
/*
 * Copyright (c) 2021, knokbak and contributors.
 *
 * The Birb.JS Project: https://birb.js.org
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ClientError_1 = __importDefault(require("../errors/ClientError"));
const ClientWarning_1 = __importDefault(require("../errors/ClientWarning"));
const OptionError_1 = __importDefault(require("../errors/OptionError"));
const GuildBlock_1 = __importDefault(require("./blocks/GuildBlock"));
const UserBlock_1 = __importDefault(require("./blocks/UserBlock"));
const Intents_1 = __importDefault(require("./Intents"));
const Websocket_1 = __importDefault(require("./websocket/Websocket"));
class Client {
    constructor(options = {}) {
        this.valid = true;
        this.token = null;
        this.ws = null;
        this.me = null;
        this.options = {
            intents: null,
            debug: false,
        };
        this.events = {};
        if ('intents' in options) {
            if (!(options.intents instanceof Intents_1.default)) {
                throw new OptionError_1.default('intents must be an instance of Intents\nMore info: https://birb.js.org/explained/intents');
            }
            if (options.intents.isEmpty()) {
                throw new OptionError_1.default('intents must have at least one flag set\nMore info: https://birb.js.org/explained/intents');
            }
        }
        else {
            throw new OptionError_1.default('intents must be provided\nMore info: https://birb.js.org/explained/intents');
        }
        this.options = Object.assign(this.options, options);
        this.guilds = new GuildBlock_1.default(this);
        this.users = new UserBlock_1.default(this, {
            maxSize: 250000,
            removeOldest: true,
        });
    }
    /**
     * Add an event listener.
     *
     * @param {EventResolvable} event The event name.
     * @param {Function} listener The function to call when the event is emitted.
     * @returns {void}
     */
    listen(event, callback) {
        if (!this.valid) {
            throw new ClientError_1.default('the client has been invalidated; please restart the process');
        }
        if (this.events[event] !== undefined) {
            process.emitWarning(new ClientWarning_1.default('binding to an event multiple times is not supported: previous listener overwritten'));
        }
        this.events[event] = callback;
    }
    /**
     * Undind an event listener.
     *
     * @param {EventResolvable} event The event name to unbind.
     * @returns {void}
     */
    unbind(event) {
        if (!this.valid)
            throw new ClientError_1.default('the client has been invalidated; please restart the process');
        delete this.events[event];
    }
    /**
     * Emit an event.
     *
     * @param {EventResolvable} event The event name.
     * @param {any} data The event data to pass to the listener.
     * @returns {void}
     */
    emit(event, ...args) {
        if (this.events[event] == undefined)
            return;
        this.events[event](...args);
    }
    /**
     * Connect to the Discord gateway.
     *
     * @param {string} token Your bot's token.
     * @returns {void}
     */
    connect(token) {
        if (!this.valid)
            throw new ClientError_1.default('the client has been invalidated; please restart the process');
        this.token = token;
        this.ws = new Websocket_1.default(this, 'gateway.discord.gg');
        this.ws.connect();
    }
    /**
     * Emit a log in debug mode.
     *
     * @param {any} message The log message.
     * @returns {void}
     */
    debug(...message) {
        if (this.options.debug)
            console.debug('[debug]', ...message);
    }
    /**
     * Emit a warning in debug mode.
     *
     * @param {any} message The warning message.
     * @returns {void}
     */
    warn(...message) {
        if (this.options.debug)
            console.warn('[warn]', ...message);
    }
    /**
     * Invalidate the client, forcing the user to restart
     * the process to use it. Once called, this cannot be
     * undone (mostly because it would defeat the entire
     * purpose of this function).
     *
     * @returns {void}
     * @public
     */
    _invalidate() {
        this.valid = false;
        this.events = {};
        console.error(new ClientError_1.default('the Birb.JS client has been invalidated; please restart the process'));
    }
}
exports.default = Client;
