import { NSFWLevel, MFALevel, NotificationLevel, VerificationLevel, ExplicitContentFilterLevel } from '../util/Constants';
import ChannelBlock from './blocks/ChannelBlock';
import RoleBlock from './blocks/RoleBlock';
import Client from './Client';
import GuildMemberBlock from './blocks/GuildMemberBlock';
import { ChannelResolvable } from '../util/Types';
export default class Guild {
    client: Client;
    readonly id: string;
    name: string;
    icon: string | null;
    nsfw: boolean;
    discoverySplash: string | null;
    banner: string | null;
    description: string | null;
    verificationLevel: VerificationLevel;
    defaultNotifications: NotificationLevel;
    available: boolean;
    systemChannelId: string | null;
    ownerId: string;
    large: boolean;
    afkChannelId: string | null;
    explicitContentFilter: ExplicitContentFilterLevel;
    maxMembers: number | null;
    preferredLocale: string | null;
    mfaLevel: MFALevel;
    vanityCode: string | null;
    nsfwLevel: NSFWLevel;
    maxVideoChannelUsers: number | null;
    approximateMemberCount: number | null;
    approximatePresenceCount: number | null;
    boosterCount: number | null;
    splash: string | null;
    memberCount: number | null;
    boostTier: number;
    rulesChannelId: string | null;
    afkTimeout: number;
    roles: RoleBlock;
    channels: ChannelBlock;
    members: GuildMemberBlock;
    constructor(client: Client, data: any);
    private build;
    /**
     * Change this Guild's name.
     *
     * @param {String} name The new name for the Guild.
     * @param {String} [reason] The reason for changing the Guild's name.
     * @returns {Promise<void>} A Promise that voids when the Guild's name has been changed.
     * @public
     */
    setName(name: string, reason?: string): Promise<void>;
    /**
     * Change this Guild's verification level.
     *
     * @param {VerificationLevel} level The new verification level for the Guild.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     * @public
     */
    setVerificationLevel(level: VerificationLevel, reason?: string): Promise<void>;
    /**
     * Change this Guild's default notifications level.
     *
     * @param {NotificationLevel} level The new notification level for the Guild.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     * @public
     */
    setDefaultNotifications(level: NotificationLevel, reason?: string): Promise<void>;
    /**
     * Change this Guild's explicit content filter level.
     *
     * @param {ExplicitContentFilterLevel} level The new explicit content filter level for the Guild.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     * @public
     */
    setExplicitContentFilter(level: ExplicitContentFilterLevel, reason?: string): Promise<void>;
    /**
     * Set this Guild's AFK timeout channel.
     *
     * @param {ChannelResolvable} channel The AFK timeout channel.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     * @public
     */
    setAfkChannel(channel: ChannelResolvable, reason?: string): Promise<void>;
    /**
     * Change this Guild's AFK timeout.
     *
     * @param {number} seconds The AFK timeout in seconds.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     * @public
     */
    setAfkTimeout(seconds: number, reason?: string): Promise<void>;
    /**
     * Set this Guild's system messages channel.
     *
     * @param {ChannelResolvable} channel The channel.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     * @public
     */
    setSystemChannel(channel: ChannelResolvable, reason?: string): Promise<void>;
    /**
     * Set this Guild's rules channel.
     *
     * @param {ChannelResolvable} channel The channel.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     * @public
     */
    setRulesChannel(channel: ChannelResolvable, reason?: string): Promise<void>;
    /**
     * Set this Guild's public updates channel.
     *
     * @param {ChannelResolvable} channel The channel.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     * @public
     */
    setPublicUpdatesChannel(channel: ChannelResolvable, reason?: string): Promise<void>;
    /**
     * Set the Guild's preferred locale.
     *
     * @param {String} locale The new locale.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     * @public
     */
    setPreferredLocale(locale: string, reason?: string): Promise<void>;
    /**
     * Set the Guild's description.
     *
     * @param {String} description The new description.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     * @public
     */
    setDescription(description: string, reason?: string): Promise<void>;
    /**
     * Toggle the Guild's booster progress bar.
     *
     * @param {boolean} toggle Whether or not to show the booster progress bar.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     * @public
     */
    setBoostProgressBar(toggle: number, reason?: string): Promise<void>;
    /**
     * Modify the Guild.
     *
     * @param {Object} data The data to send to the Discord API.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     */
    modify(data: Object, reason?: string): Promise<void>;
    /**
     * Leave the Guild.
     *
     * @returns {Promise<void>} A Promise that voids when the Guild has been left.
     */
    leave(): Promise<void>;
    /**
     * Set the Guild's data to the cache.
     *
     * @returns {Guild} This Guild instance.
     * @private
     */
    private set;
}
