"use strict";
/*
 * Copyright (c) 2021, knokbak and contributors.
 *
 * The Birb.JS Project: https://birb.js.org
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
let Zlib;
let Erlpack;
const Constants_1 = require("../../util/Constants");
const WebsocketProcessingError_1 = __importDefault(require("../../errors/WebsocketProcessingError"));
try {
    Erlpack = require('erlpack');
}
catch {
    Erlpack = null;
}
try {
    Zlib = require('zlib-sync');
}
catch {
    Zlib = null;
}
class InternalWebsocket {
    constructor(client, domain) {
        this.client = null;
        this.url = null;
        this.domain = null;
        this.status = Constants_1.Status.IDLE;
        this.encoding = Erlpack ? 'etf' : 'json';
        this.buffer = null;
        this.client = client;
        this.domain = domain;
        this.url = this.generateURL();
    }
    processPacket(data) {
        let raw;
        if (data instanceof ArrayBuffer) {
            data = new Uint8Array(data);
        }
        if (Zlib) {
            let length = data.length;
            let isFull = length >= 4 && data[length - 4] === 0x00 &&
                data[length - 3] === 0x00 && data[length - 2] === 0xff && data[length - 1] === 0xff;
            this.buffer.push(data, isFull && Zlib.Z_SYNC_FLUSH);
            if (!isFull)
                return;
            raw = this.buffer.result;
        }
        else
            raw = data;
        try {
            data = this.unpack(raw);
        }
        catch (e) {
            console.error(e);
            throw new WebsocketProcessingError_1.default('failed to unpack packet; more information is likely above');
        }
        return data;
    }
    unpack(data, type) {
        if (this.encoding === 'json' || type === 'json') {
            if (typeof data !== 'string') {
                data = (new TextDecoder()).decode(data);
            }
            return JSON.parse(data);
        }
        if (!Buffer.isBuffer(data)) {
            data = Buffer.from(new Uint8Array(data));
        }
        return Erlpack.unpack(data);
    }
    pack(data) {
        if (Erlpack)
            return Erlpack.pack(data);
        else
            return JSON.stringify(data);
    }
    init() {
        if (Zlib)
            this.buffer = new Zlib.Inflate({
                chunkSize: 65535,
                to: this.encoding === 'json' ? 'string' : undefined,
            });
    }
    generateURL() {
        let params = new URLSearchParams();
        params.set('v', '9');
        params.set('encoding', this.encoding);
        if (Zlib) {
            params.set('compress', 'zlib-stream');
        }
        return new URL(`wss://${this.domain}/?${params.toString()}`);
    }
}
exports.default = InternalWebsocket;
