/// <reference types="node" />
import Client from '../Client';
import InternalWebsocket from './InternalWebsocket';
import { WebSocket as WS } from 'ws';
import { Status } from '../../util/Constants';
export default class Websocket extends InternalWebsocket {
    ws: WS;
    ping: number;
    expectedGuilds: Set<string>;
    protected lastHeartbeat: number;
    protected lastHeartbeatAcked: boolean;
    protected heartbeatInterval: null;
    protected schedulerLoop: NodeJS.Timer;
    protected lastSequenceIdentifier: null;
    protected sessionIdentifier: string | null;
    protected doNotReconnect: boolean;
    /**
     * Creates a websocket connection to a Discord gateway.
     *
     * @param {Client} client The client.
     * @param {string} domain The domain to connect to.
     * @public
     */
    constructor(client: Client, domain: string);
    /**
     * Connect to the gateway.
     *
     * @returns {void}
     * @public
     */
    connect(): void;
    /**
     * Reconnect to the gateway.
     *
     * @returns {void}
     * @public
     */
    reconnect(): void;
    /**
     * Process a packet.
     *
     * @param {any} data The packet's data.
     * @returns {void}
     * @private
     */
    private onPacket;
    /**
     * Process the websocket opening.
     *
     * @returns {void}
     * @private
     */
    private onOpen;
    /**
     * Process the websocket closing.
     *
     * @param {number} code The closing code.
     * @returns {Promise<void>}
     * @private
     */
    private onClose;
    /**
     * Send a packet to the gateway.
     *
     * @param {Object} data The data to send.
     * @returns {void}
     * @public
     */
    send(data: Object): void;
    /**
     * Send an identify packet to the gateway.
     *
     * @returns {void}
     * @private
     */
    private identify;
    /**
     * Send a heartbeat to the gateway.
     *
     * @returns {Promise<void>}
     * @public
     */
    heartbeat(): Promise<void>;
    /**
     * Close the websocket.
     *
     * @param {number} [code] The close code.
     * @returns {void}
     * @public
     */
    close(code?: number): void;
    /**
     * Terminate the websocket. Does not notify the gateway
     * that you've disconnected. Should only be used when
     * the gateway won't respond (e.g. timeouts).
     *
     * @returns {void}
     * @public
     */
    terminate(): void;
    /**
     * Check if the websocket is connected.
     *
     * @returns {boolean}
     * @public
     */
    isConnected(): boolean;
    /**
     * Check if the websocket is closed.
     *
     * @returns {boolean}
     * @public
     */
    isClosed(): boolean;
    /**
     * Prevent reconnection.
     *
     * @param {boolean} [doNotReconnect=true] Whether or not to prevent reconnection.
     * @returns {void}
     * @public
     */
    preventReconnect(value?: boolean): void;
    /**
     * Set the websocket's status.
     *
     * @param {Status} status The status to set.
     * @returns {void}
     * @public
     */
    setStatus(status: Status): void;
    /**
     * Set the websocket's session identifier.
     *
     * @param {string} sessionIdentifier The session identifier to set.
     * @returns {void}
     * @public
     */
    setSessionIdentifier(sessionIdentifier: string): void;
    /**
     * Clean up the websocket.
     *
     * @returns {void}
     * @private
     */
    private cleanup;
    /**
     * Creates a loop that will manage repeating tasks. It
     * will call it's method every 50ms.
     *
     * @returns {void}
     * @private
     */
    private scheduler;
}
