"use strict";
/*
 * Copyright (c) 2021, knokbak and contributors.
 *
 * The Birb.JS Project: https://birb.js.org
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const InternalWebsocket_1 = __importDefault(require("./InternalWebsocket"));
const ws_1 = require("ws");
const Constants_1 = require("../../util/Constants");
const WebsocketError_1 = __importDefault(require("../../errors/WebsocketError"));
const ShardingError_1 = __importDefault(require("../../errors/ShardingError"));
class Websocket extends InternalWebsocket_1.default {
    /**
     * Creates a websocket connection to a Discord gateway.
     *
     * @param {Client} client The client.
     * @param {string} domain The domain to connect to.
     * @public
     */
    constructor(client, domain) {
        super(client, domain);
        this.ws = null;
        this.ping = 0;
        this.expectedGuilds = new Set();
        this.lastHeartbeat = 0;
        this.lastHeartbeatAcked = false;
        this.heartbeatInterval = null;
        this.schedulerLoop = null;
        this.lastSequenceIdentifier = null;
        this.sessionIdentifier = null;
        this.doNotReconnect = false;
        this.scheduler();
    }
    /**
     * Connect to the gateway.
     *
     * @returns {void}
     * @public
     */
    connect() {
        if (this.status !== Constants_1.Status.IDLE) {
            throw new WebsocketError_1.default('cannot create new websocket; already connected; run close() first');
        }
        this.status = Constants_1.Status.CONNECTING;
        this.preventReconnect(false);
        this.init();
        this.ws = new ws_1.WebSocket(this.url);
        this.ws.on('message', this.onPacket.bind(this));
        this.ws.on('open', () => this.ws.ping());
        this.ws.on('close', this.onClose.bind(this));
    }
    /**
     * Reconnect to the gateway.
     *
     * @returns {void}
     * @public
     */
    reconnect() {
        if (this.status === Constants_1.Status.IDLE) {
            this.connect();
            return;
        }
        if (this.doNotReconnect) {
            throw new WebsocketError_1.default('reconnection is not possible; restart the client\'s process');
        }
        if (this.status !== Constants_1.Status.DISCONNECTED) {
            throw new WebsocketError_1.default('cannot create new websocket; already connected; run close() first');
        }
        this.status = Constants_1.Status.RECONNECTING;
        this.init();
        this.ws = new ws_1.WebSocket(this.url);
        this.ws.on('message', this.onPacket.bind(this));
        this.ws.on('open', () => this.onOpen.bind(this));
        this.ws.on('close', this.onClose.bind(this));
    }
    /**
     * Process a packet.
     *
     * @param {any} data The packet's data.
     * @returns {void}
     * @private
     */
    onPacket(body) {
        body = this.processPacket(body);
        if (!body) {
            this.client.warn('received invalid gateway packet; ignoring it');
            return;
        }
        if (this.client.options.debug) {
            this.client.debug(`RECV: ${JSON.stringify(body)}`);
        }
        let data = body.d || {};
        if (body.s) {
            this.lastSequenceIdentifier = body.s;
        }
        switch (body.op) {
            case Constants_1.PacketOperation.DISPATCH: {
                let event = '';
                let split = body.t.split('');
                let permitted = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_';
                for (let i = 0; i < split.length; ++i) {
                    if (permitted.includes(split[i]))
                        event += split[i];
                }
                try {
                    // why js? well, ts just compiles into js, so the actual
                    // file name is [event].js once compiled into js.
                    let handler = require(`./handlers/${event}.js`);
                    this.client.debug(`DISP: received ${event} event`);
                    // oh and why handler.default()? NO CLUE TS IS WEIRD
                    handler.default(this, data);
                }
                catch (err) { }
                break;
            }
            case Constants_1.PacketOperation.HELLO: {
                if (this.status !== Constants_1.Status.CONNECTING) {
                    this.client.warn('received hello packet whilst not in connecting state; ignoring it');
                    return;
                }
                this.client.debug('discord hello');
                this.client.debug(`heartbeat interval: ${data.heartbeat_interval}ms`);
                this.heartbeatInterval = data.heartbeat_interval;
                this.lastHeartbeatAcked = true;
                this.lastHeartbeat = Date.now();
                this.ping = 0;
                this.identify();
                break;
            }
            case Constants_1.PacketOperation.HEARTBEAT_ACK: {
                this.client.debug('received heartbeat ack');
                this.lastHeartbeatAcked = true;
                this.ping = Date.now() - this.lastHeartbeat;
                this.client.debug(`ping: ${this.ping}ms`);
                break;
            }
            default: {
                this.client.warn(`received packet with op ${body.op}; ignoring it`);
                break;
            }
        }
    }
    /**
     * Process the websocket opening.
     *
     * @returns {void}
     * @private
     */
    onOpen() {
        this.client.debug('websocket opened');
        this.ws.ping();
    }
    /**
     * Process the websocket closing.
     *
     * @param {number} code The closing code.
     * @returns {Promise<void>}
     * @private
     */
    async onClose(code) {
        this.cleanup();
        this.client.debug(`websocket closed with code ${code}`);
        if (!this.isConnected())
            // the disconnection was intentional
            return;
        this.status = Constants_1.Status.DISCONNECTED;
        switch (code) {
            case Constants_1.GatewayCloseCode.UNKNOWN_ERROR: {
                this.client.warn('websocket closed due to unknown discord error');
                console.error(new WebsocketError_1.default('websocket closed due to an unknown discord error'));
                break;
            }
            case Constants_1.GatewayCloseCode.UNKNOWN_OPCODE:
            case Constants_1.GatewayCloseCode.DECODE_ERROR: {
                this.client.warn('websocket closed due to an unknown opcode or decoding error');
                console.error(new WebsocketError_1.default('websocket closed due to an unknown opcode or decoding error; is Birb.JS is up-to-date?'));
                break;
            }
            case Constants_1.GatewayCloseCode.NOT_AUTHENTICATED: {
                this.client.warn('websocket closed due to not being authenticated');
                console.error(new WebsocketError_1.default('websocket closed due to not authenticating'));
                break;
            }
            case Constants_1.GatewayCloseCode.AUTHENTICATION_FAILED: {
                this.client.warn('websocket closed due to authentication failure');
                console.error(new WebsocketError_1.default('could not authenticate with the gateway; is your token correct?'));
                this.preventReconnect(true);
                break;
            }
            case Constants_1.GatewayCloseCode.ALREADY_AUTHENTICATED: {
                this.client.warn('websocket closed due to already being authenticated');
                console.error(new WebsocketError_1.default('sent a second identify packet to the gateway; this should not be happening'));
                break;
            }
            case Constants_1.GatewayCloseCode.INVALID_SEQ:
            case Constants_1.GatewayCloseCode.SESSION_TIMEOUT: {
                this.client.warn('websocket closed due to invalid session');
                console.error(new WebsocketError_1.default('websocket closed due to an invalid session'));
                break;
            }
            case Constants_1.GatewayCloseCode.RATE_LIMITED: {
                this.client.warn('websocket closed due to rate limiting');
                console.error(new WebsocketError_1.default('you\'ve sent too many requests to the gateway; disconnected'));
                break;
            }
            case Constants_1.GatewayCloseCode.INVALID_SHARD: {
                this.client.warn('websocket closed due to invalid shard');
                console.error(new ShardingError_1.default('sharding details are invalid; disconnected from gateway'));
                this.preventReconnect(true);
                break;
            }
            case Constants_1.GatewayCloseCode.SHARDING_REQUIRED: {
                this.client.warn('websocket closed due to sharding being required');
                console.error(new ShardingError_1.default('you must shard your Discord bot due to the amount of servers it\'s in\nHead to: https://birb.js.org/explained/sharding'));
                this.preventReconnect(true);
                break;
            }
            case Constants_1.GatewayCloseCode.INVALID_VERSION: {
                this.client.warn('websocket closed due to invalid version');
                console.error(new WebsocketError_1.default('the gateway version is invalid; is Birb.JS is up-to-date?'));
                this.preventReconnect(true);
                break;
            }
            case Constants_1.GatewayCloseCode.INVALID_INTENTS: {
                this.client.warn('websocket closed due to invalid intents');
                console.error(new WebsocketError_1.default('the intents provided are invalid\nHead to: https://birb.js.org/explained/intents'));
                this.preventReconnect(true);
                break;
            }
            case Constants_1.GatewayCloseCode.DISALLOWED_INTENTS: {
                this.client.warn('websocket closed due to disallowed intents');
                console.error(new WebsocketError_1.default('you do not have authorization to use the intents provided\nHead to: https://birb.js.org/explained/intents'));
                this.preventReconnect(true);
                break;
            }
            default: {
                this.client.debug('websocket closed');
                break;
            }
        }
        if (!this.doNotReconnect) {
            await wait(15000);
            this.reconnect();
        }
        else {
            console.error(new WebsocketError_1.default('websocket closed; unable to reconnect'));
        }
    }
    /**
     * Send a packet to the gateway.
     *
     * @param {Object} data The data to send.
     * @returns {void}
     * @public
     */
    send(data) {
        this.client.debug(`SEND: ${JSON.stringify(data)}`);
        try {
            data = this.pack(data);
        }
        catch (err) {
            console.error(err);
            throw new WebsocketError_1.default('failed to package packet data');
        }
        if ([Constants_1.Status.IDLE, Constants_1.Status.CONNECTING, Constants_1.Status.RECONNECTING, Constants_1.Status.DISCONNECTED].includes(this.status)) {
            throw new WebsocketError_1.default('cannot send packet; websocket is not ready');
        }
        this.ws.send(data);
    }
    /**
     * Send an identify packet to the gateway.
     *
     * @returns {void}
     * @private
     */
    identify() {
        if (this.status === Constants_1.Status.RESUMING) {
            this.client.debug('sending resume packet...');
            this.send({
                op: Constants_1.PacketOperation.RESUME,
                d: {
                    token: this.client.token,
                    session_id: this.sessionIdentifier,
                    seq: this.lastSequenceIdentifier || null,
                },
            });
            return;
        }
        if (this.status !== Constants_1.Status.CONNECTING) {
            throw new WebsocketError_1.default('cannot send identify packet; not in connecting state');
        }
        this.client.debug('sending identify packet...');
        this.status = Constants_1.Status.IDENTIFYING;
        this.send({
            op: Constants_1.PacketOperation.IDENTIFY,
            d: {
                token: this.client.token,
                intents: this.client.options.intents.flags,
                properties: {
                    '$os': process.platform,
                    '$browser': 'birb.js',
                    '$device': 'birb.js',
                },
                compress: true,
            },
        });
    }
    /**
     * Send a heartbeat to the gateway.
     *
     * @returns {Promise<void>}
     * @public
     */
    async heartbeat() {
        if (!this.isConnected())
            // don't throw an error, it'll cause chaos
            return;
        if (this.lastHeartbeat && !this.lastHeartbeatAcked) {
            this.client.warn('heartbeat not acked (possible timeout?)');
            this.terminate();
            await wait(5000);
            this.reconnect();
        }
        this.client.debug('sending heartbeat...');
        this.lastHeartbeat = Date.now();
        this.send({
            op: Constants_1.PacketOperation.HEARTBEAT,
            d: this.lastSequenceIdentifier ?? null,
        });
    }
    /**
     * Close the websocket.
     *
     * @param {number} [code] The close code.
     * @returns {void}
     * @public
     */
    close(code) {
        if (!this.isConnected())
            // don't throw an error, it'll cause chaos
            return;
        this.ws.close(code);
        this.cleanup();
    }
    /**
     * Terminate the websocket. Does not notify the gateway
     * that you've disconnected. Should only be used when
     * the gateway won't respond (e.g. timeouts).
     *
     * @returns {void}
     * @public
     */
    terminate() {
        if (!this.isConnected())
            // don't throw an error, it'll cause chaos
            return;
        this.ws.terminate();
        this.cleanup();
    }
    /**
     * Check if the websocket is connected.
     *
     * @returns {boolean}
     * @public
     */
    isConnected() {
        return ![Constants_1.Status.IDLE, Constants_1.Status.CONNECTING, Constants_1.Status.RECONNECTING, Constants_1.Status.DISCONNECTED].includes(this.status);
    }
    /**
     * Check if the websocket is closed.
     *
     * @returns {boolean}
     * @public
     */
    isClosed() {
        return [Constants_1.Status.IDLE, Constants_1.Status.DISCONNECTED].includes(this.status);
    }
    /**
     * Prevent reconnection.
     *
     * @param {boolean} [doNotReconnect=true] Whether or not to prevent reconnection.
     * @returns {void}
     * @public
     */
    preventReconnect(value) {
        this.doNotReconnect = value ?? true;
    }
    /**
     * Set the websocket's status.
     *
     * @param {Status} status The status to set.
     * @returns {void}
     * @public
     */
    setStatus(status) {
        this.status = status;
    }
    /**
     * Set the websocket's session identifier.
     *
     * @param {string} sessionIdentifier The session identifier to set.
     * @returns {void}
     * @public
     */
    setSessionIdentifier(sessionIdentifier) {
        this.sessionIdentifier = sessionIdentifier;
    }
    /**
     * Clean up the websocket.
     *
     * @returns {void}
     * @private
     */
    cleanup() {
        if (this.schedulerLoop)
            clearInterval(this.schedulerLoop);
        this.ping = 0;
        this.lastHeartbeat = 0;
        this.lastHeartbeatAcked = false;
        this.heartbeatInterval = null;
        this.schedulerLoop = null;
    }
    /**
     * Creates a loop that will manage repeating tasks. It
     * will call it's method every 50ms.
     *
     * @returns {void}
     * @private
     */
    scheduler() {
        if (this.schedulerLoop) {
            clearInterval(this.schedulerLoop);
        }
        this.schedulerLoop = setInterval(() => {
            if (![Constants_1.Status.READY, Constants_1.Status.WAITING_FOR_GUILDS].includes(this.status)) {
                return;
            }
            if (!this.heartbeatInterval || Date.now() >= this.lastHeartbeat + this.heartbeatInterval) {
                this.heartbeat();
            }
        }, 50);
    }
}
exports.default = Websocket;
async function wait(ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
}
